#include <stdexcept>
#include <msp/strings/format.h>
#include "glcontext.h"
#include "window.h"

using namespace std;

namespace Msp {
namespace Graphics {

GLOptions::GLOptions():
	alpha(false),
	stencil(false),
	doublebuffer(true),
	multisample(0),
	forward_compatible(false),
	gl_version_major(0),
	gl_version_minor(0)
{ }


unsupported_gl_mode::unsupported_gl_mode(const GLOptions &opts):
	runtime_error(format("{ .alpha=%s, .stencil=%s, .doublebuffer=%s, .multisample=%d, .forward_compatible=%s, .gl_version=%d.%d }",
		opts.alpha, opts.stencil, opts.doublebuffer, opts.multisample, opts.forward_compatible, opts.gl_version_major, opts.gl_version_minor))
{ }



GLContext::GLContext(Window &wnd, const GLOptions &opts):
	display(wnd.get_display()),
	window(wnd)
{
	platform_init(opts);

	window.signal_resize.connect(sigc::mem_fun(this, &GLContext::window_resized));
}

#ifndef WITH_OPENGL
void GLContext::platform_init(const GLOptions &)
{
	throw runtime_error("no OpenGL support");
}

GLContext::~GLContext()
{ }

void GLContext::set_swap_interval(unsigned)
{ }

void GLContext::swap_buffers()
{ }

void GLContext::window_resized(unsigned, unsigned)
{ }
#endif

} // namespace Graphics
} // namespace Msp
